/*
 * The MIT License (MIT)
 *
 * Copyright (c) 2015-2022 Advanced Micro Devices, Inc. All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
#ifndef MIGRAPHX_GUARD_RTGLIB_CLONEABLE_HPP
#define MIGRAPHX_GUARD_RTGLIB_CLONEABLE_HPP

#include <migraphx/config.hpp>
#include <memory>

namespace migraphx {
inline namespace MIGRAPHX_INLINE_NS {

template <typename Base>
struct cloneable
{
    friend Base;

    virtual std::shared_ptr<Base> clone() = 0;

    template <typename Derived>
    struct derive : Base
    {
        friend Derived;

        std::shared_ptr<Base> clone() override
        {
            return std::make_shared<Derived>(static_cast<const Derived&>(*this));
        }
        template <typename... Args>
        derive(Args&&... args) : Base(std::forward<Args>(args)...)
        {
        }
    };

    struct share : Base, std::enable_shared_from_this<Base>
    {
        std::shared_ptr<Base> clone() override { return this->shared_from_this(); }
        template <typename... Args>
        share(Args&&... args) : Base(std::forward<Args>(args)...)
        {
        }
    };
    cloneable()                 = default;
    cloneable(const cloneable&) = default;
    cloneable& operator=(const cloneable&) = default;
    virtual ~cloneable() {}
};

} // namespace MIGRAPHX_INLINE_NS
} // namespace migraphx

#endif
