/*
 * The MIT License (MIT)
 *
 * Copyright (c) 2015-2023 Advanced Micro Devices, Inc. All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
#ifndef MIGRAPHX_GUARD_RTGLIB_LOAD_SAVE_HPP
#define MIGRAPHX_GUARD_RTGLIB_LOAD_SAVE_HPP

#include <migraphx/program.hpp>
#include <string>
#include <vector>

namespace migraphx {
inline namespace MIGRAPHX_INLINE_NS {

struct file_options
{
    std::string format = "msgpack";
};

MIGRAPHX_EXPORT program load(const std::string& filename,
                             const file_options& options = file_options{});
MIGRAPHX_EXPORT program load_buffer(const std::vector<char>& buffer,
                                    const file_options& options = file_options{});
MIGRAPHX_EXPORT program load_buffer(const char* buffer,
                                    std::size_t size,
                                    const file_options& options = file_options{});

MIGRAPHX_EXPORT void
save(const program& p, const std::string& filename, const file_options& options = file_options{});
MIGRAPHX_EXPORT std::vector<char> save_buffer(const program& p,
                                              const file_options& options = file_options{});

} // namespace MIGRAPHX_INLINE_NS
} // namespace migraphx

#endif
